@@ -1,6 +1,6 @@
 //
 //  Little cms
-//  Copyright (C) 1998-2004 Marti Maria
+//  Copyright (C) 1998-2006 Marti Maria
 //
 // Permission is hereby granted, free of charge, to any person obtaining 
 // a copy of this software and associated documentation files (the "Software"), 
@@ -23,11 +23,11 @@
 
 #include "lcms.h"
 
-// #define DEBUG 1
 
 // Transformations stuff
 // -----------------------------------------------------------------------
 
+
 // Interface
 
 cmsHTRANSFORM LCMSEXPORT cmsCreateTransform(cmsHPROFILE Input,
@@ -87,7 +87,7 @@ static icTagSignature Preview[]    = {icSigPreview0Tag,
 
 
 
-
+static volatile double GlobalAdaptationState = 0;
 
 // --------------------------------Stages--------------------------------------
 
@@ -294,18 +294,31 @@ void PrecalculatedXFORM(_LPcmsTRANSFORM p,
        register LPBYTE accum;
        register LPBYTE output;
        WORD wIn[MAXCHANNELS], wOut[MAXCHANNELS];
-       register unsigned int i, n;
-
+       unsigned int i, n;
+       
 
        accum  = (LPBYTE) in;
        output = (LPBYTE) out;
        n = Size;                    // Buffer len
+      
 
        for (i=0; i < n; i++) {
 
-       accum = p -> FromInput(p, wIn, accum);
-       cmsEvalLUT(p -> DeviceLink, wIn, wOut);
-       output = p -> ToOutput(p, wOut, output);
+         accum = p -> FromInput(p, wIn, accum);
+
+         // Try to speedup things on plain devicelinks  
+         
+         if (p ->DeviceLink ->wFlags == LUT_HAS3DGRID) {
+
+           p ->DeviceLink ->CLut16params.Interp3D(wIn, wOut, 
+                                    p ->DeviceLink -> T, 
+                                    &p ->DeviceLink -> CLut16params);
+		 }
+         else	   
+            cmsEvalLUT(p -> DeviceLink, wIn, wOut);
+      
+
+          output = p -> ToOutput(p, wOut, output);
        }
 }
 
@@ -371,6 +384,7 @@ void CachedXFORM(_LPcmsTRANSFORM p,
        register LPBYTE output;
        WORD wIn[MAXCHANNELS], wOut[MAXCHANNELS];
        register unsigned int i, n;
+	   WORD CacheIn[MAXCHANNELS], CacheOut[MAXCHANNELS];
 
 
        accum  = (LPBYTE) in;
@@ -382,25 +396,48 @@ void CachedXFORM(_LPcmsTRANSFORM p,
        ZeroMemory(wIn,  sizeof(WORD) * MAXCHANNELS);
        ZeroMemory(wOut, sizeof(WORD) * MAXCHANNELS);
 
+
+	   LCMS_READ_LOCK(&p ->rwlock);
+	       CopyMemory(CacheIn,  p ->CacheIn, sizeof(WORD) * MAXCHANNELS);
+	       CopyMemory(CacheOut, p ->CacheOut, sizeof(WORD) * MAXCHANNELS);
+	   LCMS_UNLOCK(&p ->rwlock);
+
        for (i=0; i < n; i++) {
 
        accum = p -> FromInput(p, wIn, accum);
-       
-       if (memcmp(wIn, p ->CacheIn, sizeof(WORD) * MAXCHANNELS) == 0) {
 
-            CopyMemory(wOut, p ->CacheOut, sizeof(WORD) * MAXCHANNELS);
+       
+       if (memcmp(wIn, CacheIn, sizeof(WORD) * MAXCHANNELS) == 0) {
+
+            CopyMemory(wOut, CacheOut, sizeof(WORD) * MAXCHANNELS);
        }
        else {
-
-            CopyMemory(p ->CacheIn, wIn, sizeof(WORD) * MAXCHANNELS);
-
-            cmsEvalLUT(p -> DeviceLink, wIn, wOut);
             
-            CopyMemory(p ->CacheOut, wOut, sizeof(WORD) * MAXCHANNELS);
-       }
+			// Try to speedup things on plain devicelinks  
 
+			 if (p ->DeviceLink ->wFlags == LUT_HAS3DGRID) {
+
+             p ->DeviceLink ->CLut16params.Interp3D(wIn, wOut, 
+                                    p ->DeviceLink -> T, 
+                                    &p ->DeviceLink -> CLut16params);
+		     }
+             else	   
+                  cmsEvalLUT(p -> DeviceLink, wIn, wOut);
+            
+
+            CopyMemory(CacheIn,  wIn,  sizeof(WORD) * MAXCHANNELS);
+            CopyMemory(CacheOut, wOut, sizeof(WORD) * MAXCHANNELS);
+       }
+       
        output = p -> ToOutput(p, wOut, output);
        }
+
+	   
+	   LCMS_WRITE_LOCK(&p ->rwlock);
+	       CopyMemory(p->CacheIn,  CacheIn, sizeof(WORD) * MAXCHANNELS);
+	       CopyMemory(p->CacheOut, CacheOut, sizeof(WORD) * MAXCHANNELS);
+	   LCMS_UNLOCK(&p ->rwlock);
+
 }
 
 
@@ -416,6 +453,7 @@ void CachedXFORMGamutCheck(_LPcmsTRANSFORM p,
        register LPBYTE output;
        WORD wIn[MAXCHANNELS], wOut[MAXCHANNELS];
        register unsigned int i, n;
+	   WORD CacheIn[MAXCHANNELS], CacheOut[MAXCHANNELS];
 
 
        accum  = (LPBYTE) in;
@@ -427,25 +465,35 @@ void CachedXFORMGamutCheck(_LPcmsTRANSFORM p,
        ZeroMemory(wIn,  sizeof(WORD) * MAXCHANNELS);
        ZeroMemory(wOut, sizeof(WORD) * MAXCHANNELS);
 
+	   LCMS_READ_LOCK(&p ->rwlock);
+	       CopyMemory(CacheIn,  p ->CacheIn, sizeof(WORD) * MAXCHANNELS);
+	       CopyMemory(CacheOut, p ->CacheOut, sizeof(WORD) * MAXCHANNELS);
+	   LCMS_UNLOCK(&p ->rwlock);
+
+
        for (i=0; i < n; i++) {
 
        accum = p -> FromInput(p, wIn, accum);
        
-       if (memcmp(wIn, p ->CacheIn, sizeof(WORD) * MAXCHANNELS) == 0) {
+       if (memcmp(wIn, CacheIn, sizeof(WORD) * MAXCHANNELS) == 0) {
 
-            CopyMemory(wOut, p ->CacheOut, sizeof(WORD) * MAXCHANNELS);
+            CopyMemory(wOut, CacheOut, sizeof(WORD) * MAXCHANNELS);
        }
        else {
-
-            CopyMemory(p ->CacheIn, wIn, sizeof(WORD) * MAXCHANNELS);
-
-            TransformOnePixelWithGamutCheck(p, wIn, wOut);
             
-            CopyMemory(p ->CacheOut, wOut, sizeof(WORD) * MAXCHANNELS);
+            TransformOnePixelWithGamutCheck(p, wIn, wOut);
+
+            CopyMemory(CacheIn, wIn, sizeof(WORD) * MAXCHANNELS);
+            CopyMemory(CacheOut, wOut, sizeof(WORD) * MAXCHANNELS);
        }
 
        output = p -> ToOutput(p, wOut, output);
        }
+
+	    LCMS_WRITE_LOCK(&p ->rwlock);
+	       CopyMemory(p->CacheIn,  CacheIn, sizeof(WORD) * MAXCHANNELS);
+	       CopyMemory(p->CacheOut, CacheOut, sizeof(WORD) * MAXCHANNELS);
+	    LCMS_UNLOCK(&p ->rwlock);
 }
 
 
@@ -506,6 +554,46 @@ void NC2deviceXform(_LPcmsTRANSFORM p,
 // --------------------------------------------------------------------------
 // Build a LUT based on shape-matrix method.
 
+
+// Some non-conformant gray profiles are using kTCR as L*,
+// this function converts the curve to XYZ PCS.
+
+static
+void FromLstarToXYZ(LPGAMMATABLE g, LPGAMMATABLE gxyz[3])
+{
+    int i;
+    int nPoints = 4096;
+    cmsCIELab Lab;
+    cmsCIEXYZ XYZ;
+    L16PARAMS L16;
+
+    // Setup interpolation across origin
+    cmsCalcL16Params(g ->nEntries, &L16);
+
+    // Allocate curves
+    gxyz[0] = cmsAllocGamma(nPoints);
+    gxyz[1] = cmsAllocGamma(nPoints);
+    gxyz[2] = cmsAllocGamma(nPoints);
+
+    // Transport from Lab to XYZ
+
+    for (i=0; i < nPoints; i++) {
+
+        WORD val = _cmsQuantizeVal(i, nPoints);
+        WORD w   = cmsLinearInterpLUT16(val, g->GammaTable, &L16);
+
+        Lab.L = ((double) 100.0 * w ) / 65535.0;
+        Lab.a = Lab.b = 0;
+
+        cmsLab2XYZ(NULL, &XYZ, &Lab);
+
+        // Should be same curve
+        gxyz[0] ->GammaTable[i] = (WORD) floor((65535.0 * XYZ.X) / D50X + 0.5);
+        gxyz[1] ->GammaTable[i] = (WORD) floor((65535.0 * XYZ.Y) / D50Y + 0.5);
+        gxyz[2] ->GammaTable[i] = (WORD) floor((65535.0 * XYZ.Z) / D50Z + 0.5);
+    }
+}
+
 // Monochrome version
 
 static
@@ -519,9 +607,16 @@ LPMATSHAPER cmsBuildGrayInputMatrixShaper(cmsHPROFILE hProfile)
        GrayTRC = cmsReadICCGamma(hProfile, icSigGrayTRCTag);        // Y
        cmsTakeIluminant(&Illuminant, hProfile);
 
-       Shapes[0] = cmsDupGamma(GrayTRC);
-       Shapes[1] = cmsDupGamma(GrayTRC);
-       Shapes[2] = cmsDupGamma(GrayTRC); 
+       if (cmsGetPCS(hProfile) == icSigLabData) {
+
+                // Fixup for Lab monochrome
+                FromLstarToXYZ(GrayTRC, Shapes);
+       }
+       else  {
+                Shapes[0] = cmsDupGamma(GrayTRC);
+                Shapes[1] = cmsDupGamma(GrayTRC);
+                Shapes[2] = cmsDupGamma(GrayTRC); 
+       }
        
        if (!Shapes[0] || !Shapes[1] || !Shapes[2])
               return NULL;
@@ -551,14 +646,38 @@ LPMATSHAPER cmsBuildGrayOutputMatrixShaper(cmsHPROFILE hProfile)
        LPGAMMATABLE GrayTRC, Shapes[3];
        LPMATSHAPER MatShaper;
        MAT3 Scale;
-
-       GrayTRC = cmsReadICCGammaReversed(hProfile, icSigGrayTRCTag);   // Y
+       
        cmsTakeIluminant(&Illuminant, hProfile);
 
-       Shapes[0] = cmsDupGamma(GrayTRC);
-       Shapes[1] = cmsDupGamma(GrayTRC);
-       Shapes[2] = cmsDupGamma(GrayTRC); 
-       
+       // That is a special case for non-compliant profiles. 
+
+       if (cmsGetPCS(hProfile) == icSigLabData) {
+                
+                LPGAMMATABLE Shapes1[3];
+
+                GrayTRC = cmsReadICCGamma(hProfile, icSigGrayTRCTag);
+                FromLstarToXYZ(GrayTRC, Shapes1);
+
+                // Reversing must be done after curve translation
+
+                Shapes[0] = cmsReverseGamma(Shapes1[0]->nEntries, Shapes1[0]);
+                Shapes[1] = cmsReverseGamma(Shapes1[1]->nEntries, Shapes1[1]);
+                Shapes[2] = cmsReverseGamma(Shapes1[2]->nEntries, Shapes1[2]);
+                
+                cmsFreeGammaTriple(Shapes1);
+
+       }
+       else  {
+
+                // Normal case
+
+                GrayTRC = cmsReadICCGammaReversed(hProfile, icSigGrayTRCTag);   // Y
+
+                Shapes[0] = cmsDupGamma(GrayTRC);
+                Shapes[1] = cmsDupGamma(GrayTRC);
+                Shapes[2] = cmsDupGamma(GrayTRC); 
+       }
+            
        if (!Shapes[0] || !Shapes[1] || !Shapes[2])
               return NULL;
       
@@ -590,8 +709,7 @@ LPMATSHAPER cmsBuildInputMatrixShaper(cmsHPROFILE InputProfile)
        // iluminant, scaled across GrayTRC
 
        if (cmsGetColorSpace(InputProfile) == icSigGrayData)
-       {
-             // if (dwFlags) *dwFlags |= cmsFLAGS_NOTPRECALC;
+       {             
               return cmsBuildGrayInputMatrixShaper(InputProfile);              
        }
 
@@ -806,6 +924,7 @@ void TakeConversionRoutines(_LPcmsTRANSFORM p, int DoBPC)
                              &ChromaticAdaptationMatrixOut,
 
                  DoBPC,
+                 p ->AdaptationState,
                  &p->Stage1,
                  &p->m1, &p->of1);
 
@@ -851,6 +970,7 @@ void TakeConversionRoutines(_LPcmsTRANSFORM p, int DoBPC)
                              &IlluminantProof,
                              &ChromaticAdaptationMatrixProof,
                  DoBPC,
+                 p ->AdaptationState,
                  &p->Stage1,
                  &p->m1, &p->of1);
 
@@ -868,6 +988,7 @@ void TakeConversionRoutines(_LPcmsTRANSFORM p, int DoBPC)
                              &IlluminantOut,
                              &ChromaticAdaptationMatrixOut,
                  0,
+                 0.0,
                  &p->Stage2,
                  &p->m2, &p->of2);
        }
@@ -885,9 +1006,9 @@ BOOL IsProperColorSpace(cmsHPROFILE hProfile, DWORD dwFormat, BOOL lUsePCS)
        if (Space == PT_ANY) return TRUE;
 
        if (lUsePCS)
-           return (_cmsICCcolorSpace(Space) == cmsGetPCS(hProfile));
+           return (Space == _cmsLCMScolorSpace(cmsGetPCS(hProfile)));
        else
-           return (_cmsICCcolorSpace(Space) == cmsGetColorSpace(hProfile));
+           return (Space == _cmsLCMScolorSpace(cmsGetColorSpace(hProfile)));
 }
 
 
@@ -925,6 +1046,9 @@ _LPcmsTRANSFORM AllocEmptyTransform(void)
     p -> NamedColorList = NULL;
     p -> EntryColorSpace = (icColorSpaceSignature) 0;
     p -> ExitColorSpace  = (icColorSpaceSignature) 0;
+    p -> AdaptationState = GlobalAdaptationState;
+
+	LCMS_CREATE_LOCK(&p->rwlock);
 
     return p;
 }
@@ -933,13 +1057,13 @@ _LPcmsTRANSFORM AllocEmptyTransform(void)
 // Identify whatever a transform is to be cached
 
 static
-void SetPrecalculatedTransform(_LPcmsTRANSFORM p, DWORD dwFlags)
+void SetPrecalculatedTransform(_LPcmsTRANSFORM p)
 {
-    if (dwFlags & cmsFLAGS_GAMUTCHECK && p ->GamutCheck != NULL) {
+    if ((p->dwOriginalFlags & cmsFLAGS_GAMUTCHECK) && p ->GamutCheck != NULL) {
         
         p -> xform = PrecalculatedXFORMGamutCheck;
         
-        if (!(dwFlags & cmsFLAGS_NOTCACHE)) {
+        if (!(p->dwOriginalFlags & cmsFLAGS_NOTCACHE)) {
             
             ZeroMemory(p ->CacheIn, sizeof(WORD) * MAXCHANNELS);
             TransformOnePixelWithGamutCheck(p, p->CacheIn, p ->CacheOut);        
@@ -951,7 +1075,7 @@ void SetPrecalculatedTransform(_LPcmsTRANSFORM p, DWORD dwFlags)
         
         p -> xform = PrecalculatedXFORM;
         
-        if (!(dwFlags & cmsFLAGS_NOTCACHE)) {
+        if (!(p->dwOriginalFlags & cmsFLAGS_NOTCACHE)) {
             
             ZeroMemory(p ->CacheIn, sizeof(WORD) * MAXCHANNELS);
             cmsEvalLUT(p ->DeviceLink, p->CacheIn, p ->CacheOut);        
@@ -963,7 +1087,7 @@ void SetPrecalculatedTransform(_LPcmsTRANSFORM p, DWORD dwFlags)
 
 // Transform is identified as device-link
 static 
-cmsHPROFILE CreateDeviceLinkTransform(_LPcmsTRANSFORM p, DWORD dwFlags)
+cmsHPROFILE CreateDeviceLinkTransform(_LPcmsTRANSFORM p)
 {
     
     if (!IsProperColorSpace(p->InputProfile, p->InputFormat, FALSE)) {
@@ -999,17 +1123,17 @@ cmsHPROFILE CreateDeviceLinkTransform(_LPcmsTRANSFORM p, DWORD dwFlags)
     p -> Phase2 = -1;
     p -> Phase3 = -1;
 
-    SetPrecalculatedTransform(p, dwFlags);
+    SetPrecalculatedTransform(p);
     
     p -> EntryColorSpace = cmsGetColorSpace(p -> InputProfile);
-    p -> ExitColorSpace = cmsGetPCS(p -> InputProfile);
+    p -> ExitColorSpace  = cmsGetPCS(p -> InputProfile);
     
     if (p ->EntryColorSpace == icSigRgbData ||
         p ->EntryColorSpace == icSigCmyData) {
                    
                     p->DeviceLink -> CLut16params.Interp3D = cmsTetrahedralInterp16;
     }
-
+               
     // Precalculated device-link profile is ready
     return (cmsHTRANSFORM) p;
 }
@@ -1017,14 +1141,12 @@ cmsHPROFILE CreateDeviceLinkTransform(_LPcmsTRANSFORM p, DWORD dwFlags)
 
 // Transform that includes proofing
 static
-void CreateProof(_LPcmsTRANSFORM p,                                        
-                        DWORD dwFlags,
-                        icTagSignature *ToTagPtr)
+void CreateProof(_LPcmsTRANSFORM p, icTagSignature *ToTagPtr)
 
 {
     icTagSignature ProofTag;
    
-    if (dwFlags & cmsFLAGS_SOFTPROOFING) {
+    if (p -> dwOriginalFlags & cmsFLAGS_SOFTPROOFING) {
 
       // Apr-15, 2002 - Too much profiles does have bogus content
       // on preview tag, so I do compute it by my own.
@@ -1067,7 +1189,7 @@ void CreateProof(_LPcmsTRANSFORM p,
     // Aug-31, 2001 - Too much profiles does have bogus content
     // on gamut tag, so I do compute it by my own.
 
-    if ((dwFlags & cmsFLAGS_GAMUTCHECK) && (dwFlags & cmsFLAGS_NOTPRECALC)) {
+    if ((p -> dwOriginalFlags & cmsFLAGS_GAMUTCHECK) && (p -> dwOriginalFlags & cmsFLAGS_NOTPRECALC)) {
 
                    
              p -> Gamut = _cmsComputeGamutLUT(p->PreviewProfile, p ->Intent);  
@@ -1100,8 +1222,7 @@ void CreateProof(_LPcmsTRANSFORM p,
 // Choose the adequate transform routine
 
 static
-_LPcmsTRANSFORM PickTransformRoutine(_LPcmsTRANSFORM p,                       
-                                    LPDWORD dwFlagsPtr,
+_LPcmsTRANSFORM PickTransformRoutine(_LPcmsTRANSFORM p,                                                         
                                     icTagSignature *FromTagPtr,
                                     icTagSignature *ToTagPtr)
 {
@@ -1123,11 +1244,12 @@ _LPcmsTRANSFORM PickTransformRoutine(_LPcmsTRANSFORM p,
                        !p->PreviewProfile  && 
                        p -> Intent != INTENT_ABSOLUTE_COLORIMETRIC && 
                        (p -> EntryColorSpace == icSigRgbData) && 
-                       (p -> ExitColorSpace == icSigRgbData)) {
+                       (p -> ExitColorSpace == icSigRgbData) &&
+                       !(p -> dwOriginalFlags & cmsFLAGS_BLACKPOINTCOMPENSATION)) {
 
                           // Yes... try to smelt matrix-shapers
                           p -> xform = MatrixShaperXFORM;
-                          *dwFlagsPtr |= cmsFLAGS_NOTPRECALC;
+                          p -> dwOriginalFlags |= cmsFLAGS_NOTPRECALC;
 
                           if (!cmsBuildSmeltMatShaper(p))
                           {
@@ -1224,22 +1346,25 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
        p = AllocEmptyTransform();
        if (p == NULL) return NULL;
 
-       p -> xform          = NormalXFORM;
-       p -> Intent         = nIntent;
-       p -> ProofIntent    = ProofingIntent;
-       p -> DoGamutCheck   = FALSE;
-       p -> InputProfile   = InputProfile;
-       p -> OutputProfile  = OutputProfile;
-       p -> PreviewProfile = ProofingProfile;            
-       p -> InputFormat    = InputFormat;
-       p -> OutputFormat   = OutputFormat;
-       p -> lInputV4Lab    = p ->lOutputV4Lab = FALSE;
+       p -> xform           = NormalXFORM;
+       p -> Intent          = nIntent;
+       p -> ProofIntent     = ProofingIntent;
+       p -> DoGamutCheck    = FALSE;
+       p -> InputProfile    = InputProfile;
+       p -> OutputProfile   = OutputProfile;
+       p -> PreviewProfile  = ProofingProfile;            
+       p -> InputFormat     = InputFormat;
+       p -> OutputFormat    = OutputFormat;
+       p -> dwOriginalFlags = dwFlags;
+
+       p -> lInputV4Lab     = p ->lOutputV4Lab = FALSE;
+
 
        p -> FromInput = _cmsIdentifyInputFormat(p, InputFormat);
        p -> ToOutput  = _cmsIdentifyOutputFormat(p, OutputFormat);
 
        // Null transform can be done without profiles
-       if ((dwFlags & cmsFLAGS_NULLTRANSFORM) ||
+       if ((p->dwOriginalFlags & cmsFLAGS_NULLTRANSFORM) ||
                         ((InputProfile == NULL) &&
                          (OutputProfile == NULL))) {
 
@@ -1259,7 +1384,7 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
        //  Device link are means to store precalculated transform grids.
        if (cmsGetDeviceClass(InputProfile) == icSigLinkClass) {
 
-            return CreateDeviceLinkTransform(p, dwFlags);
+            return CreateDeviceLinkTransform(p);
        }
          
        if (!IsProperColorSpace(InputProfile, InputFormat, FALSE)) {
@@ -1289,9 +1414,8 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
             return (cmsHTRANSFORM) p;
         }
 
-
         // Named color doesn't precalc anything
-        dwFlags |= cmsFLAGS_NOTPRECALC;
+        p -> dwOriginalFlags |= cmsFLAGS_NOTPRECALC;
        }
 
     
@@ -1339,7 +1463,7 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
 
        // If proofing is needed, add required tags/parameters
        if (ProofingProfile) 
-           CreateProof(p, dwFlags, &ToTag);                       
+           CreateProof(p, &ToTag);                       
        
 
        if (!cmsIsTag(OutputProfile,  ToTag)) {
@@ -1359,25 +1483,15 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
        }
 
 
-       if (dwFlags& cmsFLAGS_MATRIXINPUT)
+       if (p-> dwOriginalFlags & cmsFLAGS_MATRIXINPUT)
               FromTag = (icTagSignature)0;
 
-       if (dwFlags & cmsFLAGS_MATRIXOUTPUT)
+       if (p -> dwOriginalFlags & cmsFLAGS_MATRIXOUTPUT)
               ToTag = (icTagSignature)0;
 
-       
-       // *** uncomment this to disable fast gamut checking 
-        /*
-       if (dwFlags & cmsFLAGS_GAMUTCHECK) {
-
-             p -> DoGamutCheck = TRUE;
-             dwFlags |= cmsFLAGS_NOTPRECALC;
-       }
-       */
-
-       // *** Up to here 
+              
      
-       if (PickTransformRoutine(p, &dwFlags, &FromTag, &ToTag) == NULL) {
+       if (PickTransformRoutine(p, &FromTag, &ToTag) == NULL) {
 
           cmsDeleteTransform((cmsHTRANSFORM) p);
           return NULL;
@@ -1387,14 +1501,30 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
        TakeConversionRoutines(p, dwFlags & cmsFLAGS_BLACKPOINTCOMPENSATION);
 
      
-       if (!(dwFlags & cmsFLAGS_NOTPRECALC)) {
+       if (!(p -> dwOriginalFlags & cmsFLAGS_NOTPRECALC)) {
 
                LPLUT DeviceLink;   
                LPLUT GamutCheck = NULL;
                
-               DeviceLink = _cmsPrecalculateDeviceLink((cmsHTRANSFORM) p, dwFlags);
+
+               if (p ->EntryColorSpace == icSigCmykData &&
+                   p ->ExitColorSpace == icSigCmykData &&
+                   (dwFlags & cmsFLAGS_PRESERVEBLACK)) {
+
+                    DeviceLink = _cmsPrecalculateBlackPreservingDeviceLink((cmsHTRANSFORM) p, dwFlags);
+
+                    // Cannot be done at all?
+                    if (DeviceLink == NULL) 
+                            DeviceLink = _cmsPrecalculateDeviceLink((cmsHTRANSFORM) p, dwFlags);
+
+               }   
+               else {
+
+                    DeviceLink = _cmsPrecalculateDeviceLink((cmsHTRANSFORM) p, dwFlags);
+               }
                 
-               if (dwFlags & cmsFLAGS_GAMUTCHECK) {
+               // Allow to specify cmsFLAGS_GAMUTCHECK, even if no proofing profile is given               
+               if ((p ->PreviewProfile != NULL) && (p -> dwOriginalFlags & cmsFLAGS_GAMUTCHECK)) {
 
                    GamutCheck = _cmsPrecalculateGamutCheck((cmsHTRANSFORM) p);
                }
@@ -1402,11 +1532,15 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
                // If input colorspace is Rgb, Cmy, then use tetrahedral interpolation
                // for speed reasons (it only works well on spaces on Luma is diagonal, and 
                // not if luma is in separate channel)
-
                if (p ->EntryColorSpace == icSigRgbData ||
                    p ->EntryColorSpace == icSigCmyData) {
                    
-                    DeviceLink -> CLut16params.Interp3D = cmsTetrahedralInterp16;
+                   
+                    cmsCalcCLUT16ParamsEx(DeviceLink->CLut16params.nSamples,
+                                          DeviceLink->CLut16params.nInputs,
+                                          DeviceLink->CLut16params.nOutputs, 
+                                          TRUE, &DeviceLink->CLut16params);
+                    
                }
                
                // If this is a 8-bit transform, optimize LUT further.
@@ -1426,9 +1560,9 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
                     p ->DeviceLink = DeviceLink;
 
                     if ((nIntent != INTENT_ABSOLUTE_COLORIMETRIC) && 
-                        !(dwFlags & cmsFLAGS_NOWHITEONWHITEFIXUP))
+                        !(p -> dwOriginalFlags & cmsFLAGS_NOWHITEONWHITEFIXUP))
 
-							_cmsFixWhiteMisalignment(p);
+                            _cmsFixWhiteMisalignment(p);
 
                 }
                 else
@@ -1443,7 +1577,7 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
                 }
 
 
-              SetPrecalculatedTransform(p, dwFlags);
+              SetPrecalculatedTransform(p);
               
 
        }
@@ -1452,6 +1586,7 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
        p -> FromInput = _cmsIdentifyInputFormat(p, InputFormat);
        p -> ToOutput  = _cmsIdentifyOutputFormat(p, OutputFormat);
 
+	 
        return p;
 }
 
@@ -1498,6 +1633,10 @@ void LCMSEXPORT cmsDeleteTransform(cmsHTRANSFORM hTransform)
               cmsFreeMatShaper(p -> SmeltMatShaper);
        if (p ->NamedColorList)
               cmsFreeNamedColorList(p ->NamedColorList);
+	   if (p -> GamutCheck)
+			cmsFreeLUT(p -> GamutCheck); 
+
+	   LCMS_FREE_LOCK(&p->rwlock);
 
        free((void *) p);
 }
@@ -1533,6 +1672,32 @@ void LCMSEXPORT cmsGetAlarmCodes(int *r, int *g, int *b)
        *b = RGB_16_TO_8(AlarmB);
 }
 
+// Returns TRUE if the profile is implemented as matrix-shaper
+
+BOOL LCMSEXPORT _cmsIsMatrixShaper(cmsHPROFILE hProfile)
+{    
+    switch (cmsGetColorSpace(hProfile)) {
+
+    case icSigGrayData:
+        
+        return cmsIsTag(hProfile, icSigGrayTRCTag);
+
+    case icSigRgbData:
+
+        return (cmsIsTag(hProfile, icSigRedColorantTag) &&
+                cmsIsTag(hProfile, icSigGreenColorantTag) &&
+                cmsIsTag(hProfile, icSigBlueColorantTag) &&
+                cmsIsTag(hProfile, icSigRedTRCTag) &&
+                cmsIsTag(hProfile, icSigGreenTRCTag) &&
+                cmsIsTag(hProfile, icSigBlueTRCTag));
+
+    default:
+
+        return FALSE;
+    }
+}
+
+
 BOOL LCMSEXPORT cmsIsIntentSupported(cmsHPROFILE hProfile,
                                                 int Intent, int UsedDirection)
 {
@@ -1554,7 +1719,8 @@ BOOL LCMSEXPORT cmsIsIntentSupported(cmsHPROFILE hProfile,
         return FALSE;
        }
 
-       return (cmsIsTag(hProfile, TagTable[Intent]));       
+       if (cmsIsTag(hProfile, TagTable[Intent])) return TRUE;
+       return _cmsIsMatrixShaper(hProfile);
      }
 
      return (cmsTakeRenderingIntent(hProfile) == Intent);
@@ -1606,6 +1772,17 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateMultiprofileTransform(cmsHPROFILE hProfiles[],
         return NULL;
     }
 
+    // There is a simple case with just two profiles, try to catch it in order of getting
+    // black preservation to work on this function, at least with two profiles.
+
+    if (nProfiles == 2) {
+
+        if ((cmsGetDeviceClass(hProfiles[0]) != icSigLinkClass) && 
+            (cmsGetDeviceClass(hProfiles[1]) != icSigLinkClass)) 
+                   return cmsCreateTransform(hProfiles[0], dwInput, hProfiles[1], dwOutput, Intent, dwFlags);
+    }
+
+
     // Creates a phantom transform for latter filling
     p = (_LPcmsTRANSFORM) cmsCreateTransform(NULL, dwInput, 
                                              NULL, dwOutput, Intent, cmsFLAGS_NULLTRANSFORM);
@@ -1768,7 +1945,7 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateMultiprofileTransform(cmsHPROFILE hProfiles[],
     // All ok, store the newly created LUT   
     p -> DeviceLink   = Grid;
 
-    SetPrecalculatedTransform(p, dwFlags);
+    SetPrecalculatedTransform(p);
     
     for (i=nProfiles-1; i >= 0; --i)
         cmsDeleteTransform(Transforms[i]);
@@ -1779,7 +1956,7 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateMultiprofileTransform(cmsHPROFILE hProfiles[],
 
     if ((Intent != INTENT_ABSOLUTE_COLORIMETRIC) && 
         !(dwFlags & cmsFLAGS_NOWHITEONWHITEFIXUP))
-							_cmsFixWhiteMisalignment(p);
+                            _cmsFixWhiteMisalignment(p);
 
     return (cmsHTRANSFORM) p;
 
@@ -1792,4 +1969,14 @@ ErrorCleanup:
 }
 
 
+                       
+double LCMSEXPORT cmsSetAdaptationState(double d)
+{
+    double OldVal = GlobalAdaptationState;
 
+    if (d >= 0) 
+            GlobalAdaptationState = d;
+
+    return OldVal;
+    
+}
